@@ -10,12 +10,17 @@ use SNMP_util;
 use SNMP_Session;
 use ISG::ParseConfig;
 use RRDs;
-use Sys::Syslog;
+use Sys::Syslog qw(:DEFAULT setlogsock);
+setlogsock('unix')
+   if grep /^ $^O $/xo, ("linux", "openbsd", "freebsd", "netbsd");
 use File::Basename;
 
 # globale persistent variables for speedy
 use vars qw($cfg $probes $VERSION $havegetaddrinfo);
-$VERSION="1.12";
+$VERSION="1.18";
+
+# we want opts everywhere
+my %opt;
 
 BEGIN {
   $havegetaddrinfo = 0;
@@ -33,7 +38,7 @@ sub load_probes ($){
         die "$@\n" if $@;
 	eval '$prbs{$probe} = probes::'.$probe.'->new( $cfg->{Probes}{${probe}},$cfg );';
         die "$@\n" if $@;
-        die "Faild to load Probe $probe\n" unless defined $prbs{$probe};
+        die "Failed to load Probe $probe\n" unless defined $prbs{$probe};
     }
     return \%prbs;
 };
@@ -89,16 +94,82 @@ sub update_dynaddr ($$){
     } elsif ( -f "$file.snmp") { unlink "$file.snmp" };
         
 }
+sub sendmail ($$$){
+    my $from = shift;
+    my $to = shift;
+    my $body = shift;
+    if ($cfg->{General}{mailhost}){
+	my $smtp = Net::SMTP->new($cfg->{General}{mailhost});
+	$smtp->mail($from);
+	$smtp->to($to);
+	$smtp->data();
+	$smtp->datasend($body);
+	$smtp->dataend();
+	$smtp->quit;
+    } elsif ($cfg->{General}{sendmail} or -f "/usr/lib/sendmail"){
+	open (M, "|-") || exec (($cfg->{General}{sendmail} || "/usr/lib/sendmail"),"-f",$from,$to);
+	print M $body;
+	close M;
+    }
+}
 
-sub init_target_tree ($$$$$$); # predeclare recursive subs
-sub init_target_tree ($$$$$$) {
+sub init_alerts ($){
+    my $cfg = shift;
+    foreach my $a (keys %{$cfg->{Alerts}}) {
+	my $x = $cfg->{Alerts}{$a};
+        next unless ref $x eq 'HASH';
+	my $sub = " sub { my \$d = shift; my \$y = \$d->{$x->{type}};\n";
+	my @ops = split /\s*,\s*/, $x->{pattern};
+	$x->{length} = scalar @ops;
+        my $i = - $x->{length};
+        $sub .= "return 0 unless $x->{length} <= scalar \@\$y ;\n";
+	for (@ops) {
+	    /^(==|!=|<|>|<=|>=)(\d+(?:\.\d*)?|U|\*)(%?)$/
+		or die "ERROR Alert $a pattern entry '$_' is invalid\n";
+	    my $op = $1;
+	    my $value = $2;
+            my $perc = $3;
+            if ($value eq 'U') {
+		if ($op eq '==') {
+		    $sub .= "return 0 if defined \$y->[$i];\n";
+		} elsif ($op eq '!=') {
+		    $sub .= "return 0 unless defined \$y->[$i];\n";
+		} else {
+		    die "ERROR: invalid operator $op in connection U in Alert $a definition\n";
+		}
+	    } elsif ($value eq '*') {
+		if ($op ne '==') {
+		    die "ERROR: operator $op makes no sense with * in Alert $a definition\n";
+		} # do nothing else ...
+	    } else {
+		if ( $x->{type} eq 'loss') {
+                    die "ERROR: loss should be specified in % (alert $a pattern)\n" unless $perc eq "%";
+		    $value = $value / 100 * $cfg->{Database}{pings};
+		} elsif ( $x->{type} eq 'rtt' ) {
+		    $value /= 1000;
+		} else {
+		    die "ERROR: unknown alert type $x->{type}\n";
+		}
+		$sub .= "return 0 unless defined \$y->[$i] and \$y->[$i] $op $value;\n" ;
+	    }
+	    $i++;
+	}
+        $sub .= "return 1; }";
+	warn "### Compiling\n$sub\n" if $opt{debug};
+	$x->{sub} = eval ( $sub );
+	die "ERROR: compiling alert pattern $a ($x->{pattern}: $@\n" if $@;
+    }
+}
+
+sub init_target_tree ($$$$$$$); # predeclare recursive subs
+sub init_target_tree ($$$$$$$) {
     my $cfg = shift;
     my $probes = shift;
     my $probe = shift;
     my $tree = shift;
     my $name = shift;
     my $PROBE_CONF = shift;
-
+    my $alerts = shift;
 
     # inherit probe type from parent
     if (not defined $tree->{probe} or $tree->{probe} eq $probe){
@@ -110,24 +181,45 @@ sub init_target_tree ($$$$$$) {
 	}
     };
     
+    $tree->{alerts} = $alerts
+	if (not defined $tree->{alerts} and defined $alerts);
+
+    $alerts = $tree->{alerts};
+
+    if ($tree->{alerts}){
+	die "ERROR: no Alerts section\n"
+	    unless exists $cfg->{Alerts};
+	$tree->{alerts} = [ split /\s*,\s*/, $tree->{alerts} ];
+	$tree->{fetchlength} = 0;
+ 	foreach $a (@{$tree->{alerts}}) {
+	    die "ERROR: alert $a ($name) is not defined"
+		unless defined $cfg->{Alerts}{$a};
+	    $tree->{fetchlength} = $cfg->{Alerts}{$a}{length}
+		if $tree->{fetchlength} < $cfg->{Alerts}{$a}{length};
+	}
+    };
+
+
     $probe = $tree->{probe};
     $PROBE_CONF = $tree->{PROBE_CONF};
-   
+
     foreach my $prop (keys %{$tree}) {
     	next if $prop eq 'PROBE_CONF';
 	if (ref $tree->{$prop} eq 'HASH'){
 	    if (not -d $name) {
 		mkdir $name, 0755 or die "ERROR: mkdir $name: $!\n";
 	    };
-	    init_target_tree $cfg, $probes, $probe, $tree->{$prop}, "$name/$prop", $PROBE_CONF;
+	    init_target_tree $cfg, $probes, $probe, $tree->{$prop}, "$name/$prop", $PROBE_CONF,$alerts;
 	}
 	if ($prop eq 'host') {
 	    if (not -f $name.".rrd"){
-		RRDs::create ($name.".rrd", "--step",$cfg->{Database}{step},
+		RRDs::create
+			($name.".rrd", "--step",$cfg->{Database}{step},
 			      "DS:uptime:GAUGE:".(2*($cfg->{Database}{step})).":0:U",
 			      "DS:loss:GAUGE:".(2*($cfg->{Database}{step})).":0:".($cfg->{Database}{pings}),
-			      "DS:median:GAUGE:".(2*($cfg->{Database}{step})).":0:10",
-			      (map { "DS:ping${_}:GAUGE:".(2*($cfg->{Database}{step})).":0:10" }
+                               # 180 Seconds  is the max rtt we consider valid ... 
+			      "DS:median:GAUGE:".(2*($cfg->{Database}{step})).":0:180",
+			      (map { "DS:ping${_}:GAUGE:".(2*($cfg->{Database}{step})).":0:180" }
 			                                                  1..($cfg->{Database}{pings})),
 			      (map { "RRA:".(join ":", @{$_}) } @{$cfg->{Database}{_table}} ));
 		my $ERROR = RRDs::error();
@@ -200,23 +292,8 @@ sub enable_dynamic($$$$){
 
 	    my $mail;
             print STDERR "Sending smoke-agent for $usepath to $email ... ";
-            if ($cfg->{General}{mailhost}){
-                 eval "require Net::SMTP";
-                 unless ($@){
-                     my $smtp = Net::SMTP->new($cfg->{General}{mailhost});
-                     $smtp->mail($cfg->{General}{contact});
-                     $smtp->to($email);
-                     $smtp->data();
-                     $smtp->datasend($body);
-                     $smtp->dataend();
-                     $smtp->quit;
-                 }
-             } elsif ($cfg->{General}{sendmail} or -f "/usr/lib/sendmail"){
-        	open (M, "|-") || exec (($cfg->{General}{sendmail} || "/usr/lib/sendmail"),"-t");
-          	print M $body;
-                close M;
-             }
-             print STDERR "DONE\n";
+	    sendmail $cfg->{General}{contact},$email,$body;
+	    print STDERR "DONE\n";
         }
     }
     foreach my $prop ( keys %{$tree}) {
@@ -331,13 +408,14 @@ sub get_overview ($$$$){
            '--lower-limit','0',
 	   "DEF:median=${rrd}:median:AVERAGE",
 	   "DEF:loss=${rrd}:loss:AVERAGE",
+           "CDEF:ploss=loss,$cfg->{Database}{pings},/,100,*",
            "CDEF:dm=median,0,$max,LIMIT",
-           "CDEF:dm2=median,1.5,*",
+           "CDEF:dm2=median,1.5,*,0,$max,LIMIT",
 	   "LINE1:dm2", # this is for kicking things down a bit
 	   "LINE1:dm#$medc:median RTT avg\\:    ",
            "GPRINT:median:AVERAGE: %0.2lf %ss     ",
            "GPRINT:median:LAST:     latest RTT\\: %0.2lf %ss     ",
-   	   "GPRINT:loss:AVERAGE:    avg loss\\: %.2lf/$cfg->{Database}{pings} pkg   ",
+   	   "GPRINT:ploss:AVERAGE:    avg pkg loss\\: %.2lf %% ",
 	   "COMMENT:         $date\\j");
 	my $ERROR = RRDs::error();
 	$page .= "<div>";
@@ -391,11 +469,12 @@ sub findmax ($$) {
                 $maxmedian{$x} = $max * 1.5;
         }
         # make it a nice clean number
-        $maxmedian{$x} =~ s/^([0.]*)([1-9]).*/$1$2/;
+        $maxmedian{$x} =~ s/^([0-9.]*)([1-9]).*/$1$2/;
         $maxmedian{$x} += ( ${1}."1" + 0.0 ) ;
 
         $maxmedian{$x} = $cfg->{Presentation}{detail}{max_rtt} 
-                if $cfg->{Presentation}{detail}{max_rtt} && $maxmedian{$x} > $cfg->{Presentation}{detail}{max_rtt}        
+                if $cfg->{Presentation}{detail}{max_rtt} and
+		    $maxmedian{$x} > $cfg->{Presentation}{detail}{max_rtt}
      };
      return \%maxmedian;    
 }
@@ -475,6 +554,7 @@ sub get_detail ($$$$){
     do {
 	@median = ("DEF:median=${rrd}:median:AVERAGE",
 		   "DEF:loss=${rrd}:loss:AVERAGE",
+                   "CDEF:ploss=loss,$cfg->{Database}{pings},/,100,*",
            	   "GPRINT:median:AVERAGE:Median Ping RTT (avg %.1lf %ss) (",
                    "LINE1:median#202020"
         	   );
@@ -500,17 +580,18 @@ sub get_detail ($$$$){
         my $last = -1;
         my $swidth = $max->{$start} / $cfg->{Presentation}{detail}{height};
 	foreach my $loss (sort {$a <=> $b} keys %lc){
+            my $lvar = $loss; $lvar =~ s/\./d/g ;
 	    push @median, 
 	    (
-	     "CDEF:me$loss=loss,$last,GT,loss,$loss,LE,*,1,UNKN,IF,median,*",
-	     "CDEF:meL$loss=me$loss,$swidth,-",
-	     "CDEF:meH$loss=me$loss,0,*,$swidth,2,*,+",             
-	     "AREA:meL$loss",
-	     "STACK:meH$loss$lc{$loss}[1]:$lc{$loss}[0]"
+	     "CDEF:me$lvar=loss,$last,GT,loss,$loss,LE,*,1,UNKN,IF,median,*",
+	     "CDEF:meL$lvar=me$lvar,$swidth,-",
+	     "CDEF:meH$lvar=me$lvar,0,*,$swidth,2,*,+",             
+	     "AREA:meL$lvar",
+	     "STACK:meH$lvar$lc{$loss}[1]:$lc{$loss}[0]"
 	     );
              $last = $loss;
 	}
-	push @median, ( "GPRINT:loss:AVERAGE:avg loss\\: %.2lf/$cfg->{Database}{pings} pkg)\\l" );
+	push @median, ( "GPRINT:ploss:AVERAGE:avg pkg loss\\: %.2lf %%\\l" );
 #	map {print "$_<br/>"} @median;
     };
         # if we have uptime draw a colorful background or the graph showing the uptime
@@ -523,7 +604,7 @@ sub get_detail ($$$$){
        		           'GPRINT:duptime:LAST: %0.1lf days  (');
         	my %upt;
                 if ( defined $cfg->{Presentation}{detail}{uptime_colors}{_table} ) {
-                    for (@{$cfg->{Presentation}{detail}{_table}}) {
+                    for (@{$cfg->{Presentation}{detail}{uptime_colors}{_table}}) {
                         my ($num,$col,$txt) = @{$_};
                         $upt{$num} = [ $txt, "#".$col];
                     }
@@ -557,7 +638,8 @@ sub get_detail ($$$$){
 #	map {print "$_<br/>"} @upargs;
     };
         my @log = ();
-        push @log, "--logarithmic" if  $cfg->{Presentation}{detail}{logarithmic} and $cfg->{Presentation}{detail}{logarithmic} eq 'yes';
+        push @log, "--logarithmic" if  $cfg->{Presentation}{detail}{logarithmic} and
+	    $cfg->{Presentation}{detail}{logarithmic} eq 'yes';
 
         my @lazy =();
         @lazy = ('--lazy') if $lasthight{$start} and $lasthight{$start} == $max->{$start};
@@ -571,7 +653,7 @@ sub get_detail ($$$$){
            '--rigid',
            '--upper-limit', $max->{$start},
 	   @log,
-	   '--lower-limit',(@log ? '0.001' : '0'),
+	   '--lower-limit',(@log ? ($max->{$start} > 0.01) ? '0.001' : '0.0001' : '0'),
 	   '--vertical-label',"Seconds",
 	   '--imgformat','PNG',
 	   '--color', 'SHADEA#ffffff',
@@ -663,11 +745,73 @@ sub update_rrds($$$$$) {
         } 
         if ($prop eq 'host') {
             RRDs::update ( $name.".rrd", 
-        	   '--template',(join ":", "uptime", "loss", "median", map { "ping${_}" } 1..($cfg->{Database}{pings})),
+        	   '--template',(join ":", "uptime", "loss", "median",
+				 map { "ping${_}" } 1..($cfg->{Database}{pings})),
 	           "N:".$probes->{$probe}->rrdupdate_string($tree)
 		 );     
             my $ERROR = RRDs::error();
 	    die "ERROR: $ERROR\n" if $ERROR;
+	    # check alerts
+# disabled
+	    if ( $tree->{alerts}) {
+                $tree->{stack} = {loss=>[],rtt=>[]} unless defined $tree->{stack};
+		my $x = $tree->{stack};
+		my ($loss,$rtt) = 
+		    (split /:/, $probes->{$probe}->rrdupdate_string($tree))[1,2];
+		$loss = undef if $loss eq 'U';
+		$rtt = undef if $rtt eq 'U';
+		push @{$x->{loss}}, $loss;
+		push @{$x->{rtt}}, $rtt;
+		if (scalar @{$x->{loss}} > $tree->{fetchlength}){
+		    shift @{$x->{loss}};
+		    shift @{$x->{rtt}};
+		}
+		for (@{$tree->{alerts}}) {
+		    if ( &{$cfg->{Alerts}{$_}{sub}}($x) ){
+			# we got a match
+			my $from;
+                        my $line = "$name/$prop";
+                        my $base = $cfg->{General}{datadir};
+                        $line =~ s|^$base/||;
+                        $line =~ s|/host$||;
+                        $line =~ s|/|.|g;
+                        my $urlline = $line;
+                        $urlline =  $cfg->{General}{cgiurl}."?target=".$line;
+                        my $loss = "loss: ".join ", ",map {sprintf "%.0f %%", ( $_ || 0 ) / $cfg->{Database}{pings} * 100} @{$x->{loss}};
+                        my $rtt = "rtt: ".join ", ",map {$_ ? (sprintf "%.0f ms", $_*1000):"U" } @{$x->{rtt}}; 
+                        my $stamp = scalar localtime time;
+			if ($cfg->{Alerts}{to} =~ /^\|(.+)/) {
+			    system $1,$_,$line,$loss,$rtt,$tree->{host};	    			    
+			} else {
+			    sendmail $cfg->{Alerts}{from},$cfg->{Alerts}{to}, <<ALERT;
+To: $cfg->{Alerts}{to}
+From: $cfg->{Alerts}{from}
+Subject: [SmokeAlert] $_ on $line
+
+$stamp
+
+Got a match for alert "$_" for $urlline
+
+Pattern
+-------
+$cfg->{Alerts}{$_}{pattern}
+
+Data
+----
+$loss
+$rtt
+
+Comment
+-------
+$cfg->{Alerts}{$_}{comment}
+
+
+
+ALERT
+			}
+		    }
+		}
+	    }
 	}
     }
 }
@@ -677,7 +821,7 @@ sub get_parser () {
     my $TARGET = 
       {
        _sections => [ ( "PROBE_CONF", "/$KEY_RE/" ) ],
-       _vars     => [ qw (probe menu title note email host remark) ],
+       _vars     => [ qw (probe menu title alerts note email host remark) ],
        _mandatory=> [ qw (menu title) ],
        _doc => <<DOC,
 Each target section can contain information about a host to monitor as
@@ -685,6 +829,11 @@ well as further target sections. Most variables have already been
 described above. The expression above defines legal names for target
 sections.
 DOC
+       alerts    => {
+		     _doc => 'Comma separated list of alert names',
+		     _re => '[^\s,]+(,[^\s,]+)*',
+		     _re_error => 'Comma separated list of alert names',
+		    },
        host      => 
        {
 	_doc => <<DOC,
@@ -778,7 +927,7 @@ DOC
     my $parser = ISG::ParseConfig->new 
       (
        {
-	_sections  => [ qw(General Database Presentation Probes Targets) ],
+	_sections  => [ qw(General Database Presentation Probes Alerts Targets) ],
 	_mandatory => [ qw(General Database Presentation Probes Targets) ],
 	General    => 
 	{
@@ -821,6 +970,7 @@ DOC
 Instead of using sendmail, you can specify the name of an smtp server 
 and use perl's Net::SMTP module to send mail to DYNAMIC host owners (see below).
 DOC
+          _sub => sub { require Net::SMTP ||return "ERROR: loading Net::SMTP"; return undef; }
 	 },
 
 	 contact  => 
@@ -1185,15 +1335,16 @@ DOC
 	uptime_colors => {
 	  _table     => { _columns => 3,
 			  _doc => <<DOC,
-When monitoring a host with DYNAMIC addressing, SmokePing will keep track of how long
-the machine is able to keep the same IP address. This time is plotted as a color
-in the graphs background. SmokePing comes with a reasonable default setting,
-but you may choose to disagree. The table below
-lets you specify your own coloring
+When monitoring a host with DYNAMIC addressing, SmokePing will keep
+track of how long the machine is able to keep the same IP
+address. This time is plotted as a color in the graphs
+background. SmokePing comes with a reasonable default setting, but you
+may choose to disagree. The table below lets you specify your own
+coloring
 
 Example:
 
- Uptime Color     Legend
+ # Uptime Color            Legend
  3600    00ff00   "<1h"
  86400   0000ff   "<1d"
  604800  ff0000   "<1w"
@@ -1246,13 +1397,93 @@ DOC
 variables it wants to accept. So this expression here just defines legal variable names.'}
 				   }
 		  },
+	Alerts  => {
+		    _doc => <<DOC,
+The Alert section lets you setup loss and RTT detectors. After each
+round of polling, SmokePing will examine its data and determine which
+detectors match. Detectors are enabled per target and get inherited by
+the targets children.
 
+Detectors are not just simple thresholds which go off at first sight
+of a problem. They are configurable to detect special loss or RTT
+patterns. They let you look at a number of past readings to make a
+more educated decision on what kind of alert should be sent, or if an
+alert should be sent at all.
+
+The patterns are numbers prefixed with an operator indicating the type
+of comparison required for a match.
+
+The following RTT pattern detects if a targets RTT goes from constanty
+below 10ms to constantly 100ms and more:
+
+ old ------------------------------> new
+ <10,<10,<10,<10,<10,>10,>100,>100,>100
+
+Loss patterns work in a similar way, only that the
+loss is defined in percentage off the total of packets total sent packets.
+
+ old ------------------------------> new
+ ==0%,==0%,==0%,==0%,>20%,>20%,>=20%
+
+Apart from normal numbers, patterns can also contain the values B<*>
+which is true for all values regardless of the operator. And B<U>
+which is true for B<unknown> data together with the B<==> and B<=!> operators.
+
+A complete example
+
+ *** Alerts ***
+ to = admin\@company.xy
+ from = smokealert\@company.xy
+
+ +lossdetect
+ type = loss
+ # in percent
+ pattern = ==0%,==0%,==0%,==0%,>20%,>20%,>20%
+ comment = suddenly there is packet loss
+
+ +rttdetect
+ type = rtt
+ # in milli seconds
+ pattern = <10,<10,<10,<10,<10,<100,>100,>100,>100
+ comment = routing mesed up again ?
+
+DOC
+
+	     _sections => [ '/[^\s,]+/' ],
+	     _vars => [ qw(to from) ],
+	     _mandatory => [ qw(to from)],
+	     to => { doc => 'Either an email address to send alerts to, or the name of a
+program to execute when an alert matches. To call a program, the first character of the B<to>
+value must be a pipe symbol "|". The program will the be called whenever an alert matches,
+using the following 5 arguments: B<name-of-alert>, B<target>, B<loss-pattern>, B<rtt-pattern>, B<hostname>.',
+			_re => '(\|.+|.+@\S+)',
+			_re_error => 'put an email address or the name of a program here',
+		      },
+	     from => { doc => 'whom should alerts appear to be coming from ?',
+		       _re => '.+@\S+',
+		       _re_error => 'put an email address here',
+		      },
+	     '/[^\s,]+/' => {
+		  _vars => [ qw(type pattern comment) ],
+		  _mandatory => [ qw(type pattern comment) ],
+		  type => {
+		     _doc => 'Currently the pattern types B<rtt> and B<loss> are known',
+		     _re => '(rtt|loss)',
+                     _re_error => 'Use loss or rtt'
+			  },
+   	 	  pattern => {
+ 		     _doc => "a comma separated list of comparison operators and numbers. rtt patterns are in milli seconds, loss patterns are in percent",
+		     _re => '(==|<|<=|>|>=|!=)(\d+(?:\.\d*)?|U|\*)%?(,(?:==|<|<=|>|>=|!=)(?:\d+(?:\.\d*)?|U|\*)%?)*',
+ 		     _re_error => 'Could not parse pattern',
+		             },
+		  },
+        },
        Targets => {_doc        => <<DOC,
 The Target Section defines the actual work of SmokePing. It contains a hierarchical list
 of hosts which mark the endpoints of the network connections the system should monitor.
 Each section can contain one host as well as other sections.
 DOC
-		   _vars       => [ qw(probe menu title remark) ],
+		   _vars       => [ qw(probe menu title remark alerts) ],
 		   _mandatory  => [ qw(probe menu title) ],
 		   _sections   => [ ( "PROBE_CONF", "/$KEY_RE/" ) ],
 		   probe => { _doc => <<DOC },
@@ -1262,6 +1493,13 @@ DOC
 		   PROBE_CONF => $PROBEVARS,
 		   menu => { _doc => <<DOC },
 Menu entry for this section
+DOC
+                   alerts => { _doc => <<DOC },
+A comma separated list of alerts to check for this target. The alerts have
+to be setup in the Alerts section. Alerts are inherited by child nodes. Use
+anempty alerts definition to remove inherited alerts from the current target
+and ist childern.
+
 DOC
 		   title => { _doc => <<DOC },
 Title of the page when it is displayed.
@@ -1272,7 +1510,8 @@ DOC
 
 		   "/$KEY_RE/" => $TARGET
 		  }
-        }
+            
+      }
     );
     return $parser;
 }
@@ -1307,11 +1546,11 @@ sub demonize_me ($) {
         if (-f $pidfile ) {
             open PIDFILE, "<$pidfile";
             <PIDFILE> =~ /(\d+)/;
+            close PIDFILE;
             my $pid = $1;
             die "ERROR: I Quit! Another copy of $0 ($pid) seems to be running.\n".
               "       Check $pidfile\n"
                 if kill 0, $pid;
-            close PIDFILE;
         }
     }
     print "Daemonizing $0 ...\n";
@@ -1405,7 +1644,8 @@ sub load_cfg ($) {
         $probes = undef;
 	$probes = load_probes $cfg;
 	$cfg->{__probes} = $probes;
-      	init_target_tree $cfg, $probes, $cfg->{Targets}{probe}, $cfg->{Targets}, $cfg->{General}{datadir}, $cfg->{Targets}{PROBE_CONF}; 
+	init_alerts $cfg if $cfg->{Alerts};
+      	init_target_tree $cfg, $probes, $cfg->{Targets}{probe}, $cfg->{Targets}, $cfg->{General}{datadir}, $cfg->{Targets}{PROBE_CONF},$cfg->{Targets}{alerts}; 
     }    
 }
 
@@ -1500,17 +1740,19 @@ sub cgi ($) {
     }
 }
 
+    
 sub main ($) {
     umask 022;
     my $cfgfile = shift;
-    load_cfg $cfgfile;
-    my %opt;
-    GetOptions(\%opt, 'version', 'email', 
-                      'makepod','debug','restart', 'nodemon|nodaemon');
+    GetOptions(\%opt, 'version', 'email', ,'man','help',
+                      'makepod','debug','restart', 'nodemon|nodaemon') or pod2usage(2);
     if($opt{version})  { print "$RCS_VERSION\n"; exit(0) };
+    if($opt{man})      {  pod2usage(-verbose => 2); exit 0 };
+    if($opt{help})     {  pod2usage(-verbose => 1); exit 0 };
+    load_cfg $cfgfile;
+    if($opt{makepod})  { makepod $cfg; exit 0}; 
     if($opt{email})    { enable_dynamic $cfg, $cfg->{Targets},"",""; exit 0 };
-    if($opt{makepod})  { makepod $cfg; }; 
-    if($opt{restart})  { kill_smoke $cfg->{General}{piddir}."/smokeping.pid"; };
+    if($opt{restart})  { kill_smoke $cfg->{General}{piddir}."/smokeping.pid";};
     unless ($opt{debug} or $opt{nodemon}) {
     	if (defined $cfg->{General}{syslogfacility}) {
 		initialize_syslog($cfg->{General}{syslogfacility}, 
@@ -1662,7 +1904,8 @@ Almost all SmokePing functionality sitts in this Module.
 The programs B<smokeping> and B<smokeping.cgi> are merily
 figure heads allowing to hardcode some pathnames.
 
-If you feel like documenting what is happening within this library you are most welcome todo so.
+If you feel like documenting what is happening within this library you are
+most welcome todo so.
 
 =head1 COPYRIGHT
 
